@@ -38,7 +38,7 @@ def info(ctx):
     Show general information.
 
     Displays information about the attached YubiKey such as serial number,
-    firmware version, capabilities, etc.
+    firmware version, applications, etc.
     """
     dev = ctx.obj['dev']
     click.echo('Device type: {}'.format(dev.device_name))
@@ -50,18 +50,19 @@ def info(ctx):
     else:
         click.echo('Firmware version: Uncertain, re-run with only one '
                    'YubiKey connected')
-    click.echo('Enabled connection(s): {}'.format(dev.mode))
+    if dev.form_factor:
+        click.echo('Form factor: {}'.format(str(dev.form_factor)))
+    click.echo('Enabled USB interfaces: {}'.format(dev.mode))
     click.echo()
 
-    click.echo('Device capabilities:')
+    click.echo('Applications:')
     for c in CAPABILITY:
-        if c != CAPABILITY.NFC:  # For now, don't expose NFC Capability
-            if c & dev.capabilities:
-                if c & dev.enabled:
-                    status = 'Enabled'
-                else:
-                    status = 'Disabled'
+        if c & dev.capabilities:
+            if c & dev.enabled:
+                status = 'Enabled'
             else:
-                status = 'Not available'
+                status = 'Disabled'
+        else:
+            status = 'Not available'
 
-            click.echo('    {0.name}:\t{1}'.format(c, status))
+        click.echo('    {0!s}:\t{1}'.format(c, status))
