@@ -27,16 +27,20 @@
 
 from __future__ import absolute_import
 
-from .util import CAPABILITY, TRANSPORT, YUBIKEY, parse_tlvs
+from .util import CAPABILITY, TRANSPORT, YUBIKEY, FORM_FACTOR, parse_tlvs
 from .driver import AbstractDriver
 from binascii import b2a_hex
+import logging
 import six
 
 
+logger = logging.getLogger(__name__)
+
+
 YK4_CAPA_TAG = 0x01
 YK4_SERIAL_TAG = 0x02
 YK4_ENABLED_TAG = 0x03
-
+YK4_FORMFACTOR_TAG = 0x04
 
 _NULL_DRIVER = AbstractDriver()
 
@@ -57,11 +61,25 @@ class YubiKey(object):
         self._descriptor = descriptor
         self._driver = driver
         self.device_name = descriptor.key_type.value
+        self._form_factor = FORM_FACTOR.UNKNOWN
 
         if driver.key_type == YUBIKEY.SKY:
-            self.capabilities = CAPABILITY.U2F
+            try:
+                # Only new SKYs report capabilities
+                resp = driver.read_capabilities()
+                if (resp):
+                    self._parse_capabilities(resp)
+                else:
+                    self.capabilities = CAPABILITY.U2F
+            except Exception:
+                self.capabilities = CAPABILITY.U2F
+            if self.version >= (5, 0, 0):
+                self.device_name = 'Security Key by Yubico'  # SKY 2
             self._can_mode_switch = False
         elif self.version >= (4, 1, 0):
+            if (5, 1, 0) > self.version >= (5, 0, 0) and (
+                        driver.key_type == YUBIKEY.YK4):
+                self.device_name = 'YubiKey Preview'
             if self.version == (4, 2, 4):  # 4.2.4 doesn't report correctly.
                 capabilities = b'\x03\x01\x01\x3f'
             else:
@@ -83,7 +101,7 @@ class YubiKey(object):
                 # Assume base capabilities for NEO
                 self.capabilities = CAPABILITY.OTP | CAPABILITY.CCID | \
                     CAPABILITY.OPGP | CAPABILITY.PIV | CAPABILITY.OATH
-            if TRANSPORT.has(self.mode.transports, TRANSPORT.U2F) \
+            if TRANSPORT.has(self.mode.transports, TRANSPORT.FIDO) \
                     or self.version >= (3, 3, 0):
                     self.capabilities |= CAPABILITY.U2F
         else:  # Standard
@@ -109,6 +127,9 @@ class YubiKey(object):
                 self._serial = int(b2a_hex(tlv.value), 16)
             elif YK4_ENABLED_TAG == tlv.tag:
                 self.enabled = int(b2a_hex(tlv.value), 16)
+            elif YK4_FORMFACTOR_TAG == tlv.tag:
+                self._form_factor = FORM_FACTOR.from_code(
+                    int(b2a_hex(tlv.value), 16))
 
     @property
     def version(self):
@@ -122,6 +143,10 @@ class YubiKey(object):
     def serial(self):
         return self._serial or self._driver.serial
 
+    @property
+    def form_factor(self):
+        return self._form_factor
+
     @property
     def driver(self):
         return self._driver
@@ -180,6 +205,15 @@ class YubiKey(object):
         assert dev.mode == my_mode
         return dev
 
+    def close(self):
+        self._driver.close()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
     def __str__(self):
         return '{0} {1[0]}.{1[1]}.{1[2]} {2} [{3.name}]' \
             'serial: {4} CAP: {5:x}' \
