@@ -56,8 +56,8 @@ INS_NEO_TEST = 0x16
 
 KNOWN_APPLETS = {
     AID.OTP: CAPABILITY.OTP,
-    b'\xa0\x00\x00\x06\x47\x2f\x00\x01': CAPABILITY.U2F,  # Official
-    b'\xa0\x00\x00\x05\x27\x10\x02': CAPABILITY.U2F,  # Yubico - No longer used
+    AID.U2F: CAPABILITY.U2F,
+    AID.U2F_YUBICO: CAPABILITY.U2F,
     AID.PIV: CAPABILITY.PIV,
     AID.OPGP: CAPABILITY.OPGP,
     AID.OATH: CAPABILITY.OATH
@@ -87,6 +87,9 @@ def _pid_from_name(name):
         if t.name in name:
             transports += t
 
+    if 'U2F' in name:
+        transports += TRANSPORT.FIDO
+
     key_type = YUBIKEY.NEO if 'NEO' in name else YUBIKEY.YK4
     return key_type.get_pid(transports)
 
@@ -100,14 +103,24 @@ class CCIDDriver(AbstractDriver):
     def __init__(self, connection, name):
         self._conn = connection
         self._pid = _pid_from_name(name)
+        try:
+            self._read_version()
+        except APDUError as e:
+            logger.error('Failed to read firmware version', exc_info=e)
+            # Failure here likely indicates YK preview, in which case we'll
+            # later read the version via capabilities. This might be wrong and
+            # should be fixed later.
+            self._version = (5, 0, 0)  # TODO:Properly detect version
         try:
             self._read_serial()
-        except APDUError:
-            pass  # Can't read serial or version
+        except APDUError as e:
+            logger.error('Failed to read serial number', exc_info=e)
 
-    def _read_serial(self):
+    def _read_version(self):
         s = self.send_apdu(0, INS_SELECT, 4, 0, AID.OTP)
         self._version = tuple(c for c in six.iterbytes(s[:3]))
+
+    def _read_serial(self):
         serial = self.send_apdu(0, INS_YK2_REQ, SLOT_DEVICE_SERIAL, 0)
         if len(serial) == 4:
             self._serial = struct.unpack('>I', serial)[0]
@@ -120,7 +133,8 @@ class CCIDDriver(AbstractDriver):
             self.send_apdu(0, INS_SELECT, 4, 0, AID.MGR)
             capa = self.send_apdu(0, INS_YK4_CAPABILITIES, 0, 0)
             return capa
-        except APDUError:
+        except APDUError as e:
+            logger.error('Failed to read capabilities', exc_info=e)
             return b''
 
     def probe_capabilities_support(self):
@@ -129,7 +143,11 @@ class CCIDDriver(AbstractDriver):
             try:
                 self.send_apdu(0, INS_SELECT, 4, 0, aid)
                 capa |= code
+                logger.debug(
+                    'Found applet: aid: %s , capability: %s', aid, code)
             except APDUError:
+                logger.debug(
+                    'Missing applet: aid: %s , capability: %s', aid, code)
                 pass
         return capa
 
@@ -174,11 +192,16 @@ class CCIDDriver(AbstractDriver):
         self.send_apdu(0, INS_SELECT, 4, 0, AID.MGR)
         self.send_apdu(0, INS_NEO_TEST, SLOT_DEVICE_CONFIG, 0, mode_data)
 
+    def close(self):
+        logger.debug('Close %s', self)
+        self._conn.disconnect()
+
     def __del__(self):
+        logger.debug('Destroy %s', self)
         try:
-            self._conn.disconnect()
-        except Exception:
-            pass  # Ignore
+            self.close()
+        except Exception as e:
+            logger.debug('Exception in destructor', exc_info=e)
 
 
 def _pgm_seq_ok(pgm_seq_old, pgm_seq_new):
@@ -231,17 +254,17 @@ def open_devices(name_filter='yubico yubikey'):
     while readers:
         try_again = []
         for reader in readers:
-            try:
-                conn = reader.createConnection()
-                conn.connect()
-                yield CCIDDriver(conn, reader.name)
-            except CardConnectionException:
-                try_again.append(reader)
-            except Exception as e:
-                # Try with next reader.
-                logger.debug('Failed to connect to reader %s', str(reader),
-                             exc_info=e)
-
+            if reader.name.lower().startswith(name_filter):
+                try:
+                    conn = reader.createConnection()
+                    conn.connect()
+                    yield CCIDDriver(conn, reader.name)
+                except CardConnectionException:
+                    try_again.append(reader)
+                except Exception as e:
+                    # Try with next reader.
+                    logger.debug(
+                        'Failed to connect to reader %s', reader, exc_info=e)
         if try_again and kill_scdaemon():
             readers = try_again
         else:
